<?php

require_once 'base/FpaferiasEntityBase.php';


/**
 * FpaferiasEntity.
 *
 * @package Entity
 *
 * @author  alessandrobartels
 *
 */
class FpaferiasEntity extends FpaferiasEntityBase
{

    private $_nrOrg;
    private $_nrOrgPadrao; /* Organizacao Padrao */
    /** @var $_mesCompetencia TkDateTime */
    private $_mesCompetencia;
    /** @var $_gozosFerias FpagozoferiasEntity[] */
    private $_gozosFerias;

    private $_situacao_funcional; // Tem que ser tirado daqui, ferias nao tem situacao funcional. jorgefonseca
    private $_falta_intervalo; // Nao esta sendo usado (verificar).
    private $_falta_ferias;
    private $_tipo_ferias;
    private $_nrTpFeriasAnterior;
    private $_periodo_fechado; // Nao esta sendo usado (verificar).
    private $_nova_data_base;
    private $_vai_perder_periodo;
    private $_qtd_meses_periodo_aquisitivo;
    private $_dias_limite_afastamento;


    /**
     * Construtor.
     *
     * @param integer $id          Identificador.
     * @param integer $nrorg       Numero da Organizacao.
     * @param integer $nrOrgPadrao Numero da Organizacao Padrao.
     *
     * @author alessandrobartels
     *         alessandrobartels - 05/07/2011 - Parametrizacao da Org. Padrao.
     */
    public function __construct($id = null, $nrorg = null, $nrOrgPadrao = 0, $mesCompetencia = null){

        parent::__construct($id);

        $this->_nrOrg = $nrorg;
        $this->_nrOrgPadrao = $nrOrgPadrao;
        $this->_mesCompetencia = $mesCompetencia;

        $this->_nrTpFeriasAnterior = -1;
        $this->_qtd_meses_periodo_aquisitivo = 0;
        $this->_dias_limite_afastamento = 0;

        $this->_tipo_ferias = new FpatipoferiasmEntity();
        $this->_falta_ferias = new FpafaltaferiasEntity();
        try{
            $this->_situacao_funcional = TkDI::getContainer()->situacao_funcional;
//            $this->_gozo_ferias = TkDI::getContainer()->gozo_ferias; // JORGEFONSECA desnecessario
        } catch (Exception $e){
            // do nothing...
        }
    }


    /**
     * Adiciona objetos da classe em um lista.
     *
     * @param integer $nr_org     Numero da Organizacao.
     * @param integer $nr_vinculo Numero do Vinculo.
     *
     * @return void
     *
     * @author alessandrobartels
     */
    public function retornaFerias($nr_org, $nr_vinculo){
        $this->resetItems();

        // Setando vinculo corrente.
        $this->setNrvinculom($nr_vinculo);
        $this->setNrorg($nr_org);

        $resultado = Fpaferias::retornaItem($nr_vinculo, $nr_org);
        if (count($resultado) > 0) {
            foreach ($resultado as $entity_obj) {
                /* $this->addItem($entity_obj, $entity_obj->getNrferias()); */
                $this->addItem($entity_obj);
            }
        }
    }


    /**
     * Procura objeto de ferias na lista.
     *
     * @param integer $nr_ferias Numero de Ferias.
     *
     * @return FpaferiasEntity
     *
     * @author luistavares - 20/06/2011 - Adequa��o do m�todo para o padr�o do CodeSniffer.
     */
    public function procuraFerias($nr_ferias){
        $itens = $this->getItems();
        //BORBA 07/02/2013 Testa null para atender chamada para calculo de férias coletivas
        if(is_null($nr_ferias)){
            sort($itens,0);
            foreach ($itens as $indice => $item) {
                $id_control_fer = $item->getIdcontrolferias();
                if ($id_control_fer == FpaferiasEntity::IDCONTROLFERIAS_ADQUIRIDO
                    || $id_control_fer == FpaferiasEntity::IDCONTROLFERIAS_ABERTO) {
                    return $item;
                }
            }
        }
        else{
            foreach ($itens as $indice => $item) {
                $nr_ferias_item = $item->getNrFerias();
                if ((int)$nr_ferias_item === (int)$nr_ferias) {
                    return $item;
                }
            }

            //@todo MsgCalculo
            MsgCalculo::getInstance()->adicionaMensagem('Periodo aquisitivo de ferias nao encontrado: '.$nr_ferias);
            //echo '<<FPAFERIASENTITY>> Periodo aquisitivo de ferias nao encontrado: '.$nr_ferias.'<br/>';
        }
    }


    /**
     * Pesquisa a existencia de periodo de ferias.
     *
     * @param integer    $nr_tipoferias Numero do Tipo de Ferias.
     * @param TkDateTime $data_ini_per  Data do inicio do periodo.
     *
     * @return FpaferiasEntity | boolean
     *
     * @author luistavares - 20/06/2011 - Adequa��o do m�todo para o padr�o do CodeSniffer
     */
    private function _retornaPeriodoFeriasSeExiste($nr_tipoferias, TkDateTime $data_ini_per) {

        $itens = $this->getItems();
        $dt_inicio_aquisicao = new TkDateTime();
        foreach ($itens as $item) {
            $nr_tpferias = $item->getNrtipoferias();
            $dt_inicio_aquisicao->set($item->getDtiniaquisicao(), TkDateTime::FORMAT_DATE);
            if (((int)$nr_tpferias === (int)$nr_tipoferias)
                && ($dt_inicio_aquisicao->equals($data_ini_per, TkDateTime::FORMAT_DATE) === true)
            ) {
//                return $item->getNrferias();
                return $item;
            }
        }

        return false;
    }


    /**
     * Determina a data base de ferias do vinculo.
     *
     * @param GpevinculomEntity $vinculo Instancia de GpevinculomEntity.
     *
     * @return string
     *
     * @author - alessandrobartels - 22/06/2011
     */
    private function _determinaDataBaseFerias(GpevinculomEntity $vinculo){
        $dataBase = $vinculo->historico->getDtbaseferias();

        if (empty($dataBase) === true) {
            // E so uma garantia pois a data base devera ser preenchida no cadastramento.
            $dataBase = $vinculo->getDtadmissaovinc();
        }

        return $dataBase;
    }


    /**
     * Prepara atributos de ferias para calculo de Rescisao.
     *
     * @param GpevinculomEntity $vinculo                 Objeto Vinculo.
     * @param TkDateTime        $data_rescisao_projetada Data de rescisao projetada.
     *
     * @return array multitype:
     *
     * @author alessandrobartels
     */
    public function verificaFeriasRescisao(
        GpevinculomEntity $vinculo, TkDateTime $data_rescisao_projetada, boolean $primeira_vez,
        $evento, $faltasIntervalo, $jornadaDiaria,$nrTipoModalidadeCalc, $nrOcorrenciaCalc,
        GpealtesitufuncEntity $alteracao_situacao_funcional,$mesCompetencia
    ){

        /*O método é chamado duas vezes, 1ª com a data real da rescisão e a segunda com a data projetada pelas indenizações*/
        if ($primeira_vez) {
            //BORBA 10/12/2013
            /* Testa se o periodo aquisitivo corrente ja existe e faz a insercao caso contrario
            * Se for gerar um novo periodo verifica faltas e afastamentos do periodo anterior
            */
            $objTpFeriasVinc = \FpaTpferiasvincEntity::retornaTipoFeriasPorTipoVinculo(
                $this->_nrOrg, $this->_nrOrgPadrao, $vinculo->getNrtipovinculom()
            );

            $this->retornaFerias($vinculo->getNrorg(), $vinculo->getNrvinculom());

            $this->verificaPeriodoAquisitivo(
                $vinculo, $evento, $alteracao_situacao_funcional,$faltasIntervalo, $objTpFeriasVinc,
                $mesCompetencia, $jornadaDiaria,$nrTipoModalidadeCalc, $nrOcorrenciaCalc
            );

        }

        $ferias = array();
        $i = 0;

        $data_ini_per = new TkDateTime();
        $data_fim_per = new TkDateTime();

        foreach ($this->getItems() as $item) {
            $diasGozados=0;
            if (($item->getIdcontrolferias() === FpaferiasEntity::IDCONTROLFERIAS_GOZADO_PARCIAL)
                ||($item->getIdcontrolferias() === FpaferiasEntity::IDCONTROLFERIAS_ADQUIRIDO)
            ) {
                //BORBA 08/12/2013 Para trazer $diasFaltas corretamente é necessario setar o primeiro dia da competencia
                //para o $this->_totalizaDiasFaltas
                $data_ini_per_aux= new TkDateTime($item->getDtiniaquisicao());

                //BORBA 28/11/2013
                $diasFaltas = $this->_totalizaDiasFaltas(
                    $vinculo, $evento, $faltasIntervalo, $data_ini_per_aux->setDay(1),
                    $item->getDtfimaquisicao(), $jornadaDiaria, $nrTipoModalidadeCalc,
                    $nrOcorrenciaCalc
                );
                $falta_ferias = $this->_falta_ferias->verificaFaltaFerias($item->getNrtipoferias(), $diasFaltas);
                if ($falta_ferias === null) {
                    // Perda do periodo de ferias por faltas.
                    /* @todo Com zero nao atualiza */
                    $item->setQtdiasmaxfe(0);
                    $item->setQtdiasmaxabon(0);
                    $item->setIdcontrolferias(self::IDCONTROLFERIAS_PERDIDO);
					$qtd_dias = 0;
                } else {
                    $item->setQtdiasmaxfe($falta_ferias->getQtdiasmaxfe());
                    $item->setQtdiasmaxabon($falta_ferias->getQtdiasmaxabon());
                    //BORBA 28/11/2013 antes era passado $item->getQtdiasmaxfe()
                    $qtd_dias = $falta_ferias->getQtdiasmaxfe();					
                }

                $qtd_doze_avos = 0;
                $ferias[$i]['DTFIMAQUISICAO'] = $item->getDtfimaquisicao();
                /*05/03/2015 BORBA - Corrige o gozo caso exista um gozo anterior para o mesmo período*/
                if($item->getIdcontrolferias() == FpaferiasEntity::IDCONTROLFERIAS_GOZADO_PARCIAL){
                    $diasGozados=0;
                    /** @var $itemGozo FpagozoferiasEntity */
                    foreach ($item->getGozoFerias() as $itemGozo) {
                        /*getDiasGozados traz datas de inicio e fim inclusive*/
                        $diasGozados += $itemGozo->getDiasGozados();
                           //tk_warning($diasGozados.'  '.$itemGozo->getDtinigozoferias().'  '.$itemGozo->getDtfimgozoferias());
                    }
                }

            } elseif ($item->getIdcontrolferias() === FpaferiasEntity::IDCONTROLFERIAS_ABERTO) {
                $data_ini_per->set($item->getDtiniaquisicao(), TkDateTime::FORMAT_DATE);
                $data_fim_per->set($data_ini_per, TkDateTime::FORMAT_DATE);
                //BORBA 08/12/2013 Para trazer $diasFaltas corretamente é necessario setar o primeiro dia da competencia
                //para o $this->_totalizaDiasFaltas
                $data_ini_per_aux = clone $data_ini_per;

                //BORBA 28/11/2013
                $diasFaltas = $this->_totalizaDiasFaltas(
                    $vinculo, $evento, $faltasIntervalo, $data_ini_per_aux->setDay(1),
                    $item->getDtfimaquisicao(), $jornadaDiaria, $nrTipoModalidadeCalc,
                    $nrOcorrenciaCalc
                );

                $qtd_doze_avos = $data_ini_per->months($data_rescisao_projetada);
                $falta_ferias = $this->_falta_ferias->verificaFaltaFerias($item->getNrtipoferias(), $diasFaltas);
                if ($falta_ferias === null) {
                    // Perda do periodo de ferias por faltas.
                    /* @todo Com zero nao atualiza */
                    $item->setQtdiasmaxfe(0);
                    $item->setQtdiasmaxabon(0);
                    $qtd_doze_avos = 0;
					$qtd_dias = 0;
                    $item->setIdcontrolferias(self::IDCONTROLFERIAS_PERDIDO);
                } else {
                    $item->setQtdiasmaxfe($falta_ferias->getQtdiasmaxfe());
                    $item->setQtdiasmaxabon($falta_ferias->getQtdiasmaxabon());					
                    //BORBA 28/11/2013 antes era passado 0(zero)
                    $qtd_dias = $falta_ferias->getQtdiasmaxfe();					
                }
                //tk_warning($qtd_dias.'D | F'.$diasFaltas.''.$item->getDtiniaquisicao().''.$item->getDtfimaquisicao());
                $data_fim_per->addMonth($qtd_doze_avos);
                if (($data_fim_per->equals($data_rescisao_projetada, TkDateTime::FORMAT_DATE) === true)
                    ||($data_fim_per->isEarlier($data_rescisao_projetada, TkDateTime::FORMAT_DATE) === true)
                ) {
				    /*BORBA 28/11/2014 Atendendo a solicitação 155226, deduzindo faltas da qtde minima de dias*/
					/*Marcelo 15/12/2014 Retirado o desconto das faltas a pedido do pessoal do RH*/
                    $dias = $data_fim_per->daysAfter($data_rescisao_projetada);
                    if ($dias >= 15) {
                        $qtd_doze_avos++;
                    }
                }

                $ferias[$i]['DTFIMAQUISICAO'] = $data_rescisao_projetada;
            } else {
                continue;
            }

            $ferias[$i]['TIPOFERIAS'] = $item->getNrtipoferias();
            $ferias[$i]['IDCONTROLFERIAS'] = $item->getIdcontrolferias();
            $ferias[$i]['DTINIAQUISICAO'] = $item->getDtiniaquisicao();
            $ferias[$i]['QTDIASMAXFE'] = $qtd_dias;
            $ferias[$i]['QTMESESMAXFE'] = $qtd_doze_avos;
            $ferias[$i]['QTDDIASGOZADOS']= $diasGozados;
            $i++;
        }

        return $ferias;
    }


    /**
     * Atualiza a data base de ferias do vinculo.
     *
     * @param TkDateTime        $data_ini_per Data inicio do periodo de ferias.
     * @param TkDateTime        $data_fim_per Data fim do periodo de ferias.
     * @param GpevinculomEntity $vinculo      Instancia de GpevinculomEntity.
     *
     * @return void
     *
     * @author alessandrobartels - 22/06/2011
     */
    private function _atualizaDataBaseFeriasVinculo(
        TkDateTime $data_ini_per, TkDateTime $data_fim_per, GpevinculomEntity $vinculo
    ){
        if ($this->_vai_perder_periodo === true) {
            // Atualiza o proximo periodo para a data base de ferias do vinculo.
            if ($this->_nova_data_base->equals(TkDateTime::DUMMY_DATE, TkDateTime::FORMAT_DATE) === false) {
                $data_ini_per->set($vinculo->historico->getDtbaseferias());
                $data_fim_per->set($data_ini_per)->addYear(1)->subDay(1);
            }
        }
    }


    /**
     * Verifica periodo aquisitivo.
     *
     * @param GpevinculomEntity      $vinculo                      Objeto Vinculo.
     * @param FpaeventomEntity       $evento                       Objeto Evento.
     * @param GpealtesitufuncEntity  $alteracao_situacao_funcional Objeto Alteracao da Situacao Funcional.
     * @param EventosFaltasIntervalo $faltas_intervalo             Objeto Eventos Faltas Intervalo.
     * @param array                  $objTpFeriasVinc              Array de instancias de FpaTpferiasvincEntity.
     * @param TkDateTime             $mes_competencia              Mes de competencia.
     * @param integer                $jornada_diaria               Valor da jornada diaria.
     * @param integer                $nrTipoModalidadeCalc         Numero do Tipo de Modalidade de Calculo.
     * @param integer                $nrOcorrenciaCalc             Numero de Ocorrencia de Calculo.
     *
     * @return void
     *
     * @author  alessandro bartels -
     * 			luistavares - 20/06/2011 - Adequa��o do m�todo para o padr�o do CodeSniffer.
     * 			alessandrobartels - 22/06/2011 - Retirado cria��o e atualiza��o de data base deferias e passado para
     * 											 novo m�todo.
     * 			alessandrobartels - 30/11/2011 - Criado metodo _verificaExistenciaPeriodoFerias() para diminuir a
     * 											 complexidade ciclomatica.
     *          alessandrobartels - 13/02/2012 - Parametrizado $nrTipoModalidadeCalc, $nrOcorrenciaCalc.
     */
    public function verificaPeriodoAquisitivo(
        GpevinculomEntity $vinculo, FpaeventomEntity $evento, GpealtesitufuncEntity $alteracao_situacao_funcional,
        EventosFaltasIntervalo $faltas_intervalo, array $objTpFeriasVinc, TkDateTime $mes_competencia, $jornada_diaria,
        $nrTipoModalidadeCalc, $nrOcorrenciaCalc
    ){

        $data_base = $this->_determinaDataBaseFerias($vinculo);

        foreach ($objTpFeriasVinc as $tpFeriasVinc) {
            $nr_tipoferias = $tpFeriasVinc->getNrtipoferias();

            if ($this->_nrTpFeriasAnterior !== $nr_tipoferias) {
                $obj_tipo_ferias = $this->_tipo_ferias->retornaTipoFerias(
                    $this->_nrOrgPadrao, $this->_nrOrg, $nr_tipoferias
                );
                $this->_nrTpFeriasAnterior = $nr_tipoferias;
                $this->_tipo_ferias = $obj_tipo_ferias;

                if (is_object($obj_tipo_ferias) === false) {
                    //@todo MsgCalculo
                    MsgCalculo::getInstance()->adicionaMensagem('Tipo de férias não encontrado. Verificar Tipo Férias x Tipo Vínculo.');
                    //tk_warning('Tipo de ferias nao encontrado. Verificar Tipo Ferias x Tipo Vinculo.');
                    return;
                }

                $this->_qtd_meses_periodo_aquisitivo = $this->_tipo_ferias->retornaQtdMesesPeriodoAquisitivo(
                    $obj_tipo_ferias
                );

                $this->_dias_limite_afastamento = $obj_tipo_ferias->historico->getQtmesafastperda() * 30;
            }

            $data_ini_per = new TkDateTime($data_base, TkDateTime::FORMAT_DATE);
            $data_fim_per = new TkDateTime($data_ini_per, TkDateTime::FORMAT_DATE);
            // Soma o periodo e volta 1 dia para determinar o fim do periodo aquisitivo.
            $data_fim_per->addMonth($this->_qtd_meses_periodo_aquisitivo)->subDay(1);

            if ($mes_competencia->isEarlier($data_ini_per, TkDateTime::MONTH.'/'.TkDateTime::YEAR) === true) {
                /* So aconteceria num processamento no passado com alteracao de data base futura.
                 * sem geracao de historico de vinculo na data.
                 */

                $msg = '<FPAFERIASENTITY> Mes de competencia menor que Data de inicio do periodo de ferias. ';
                $msg .= 'Em principio isso nao deve ocorrer.<br/> Vinculo: '.$vinculo->getNrvinculom();
                //@todo MsgCalculo
                MsgCalculo::getInstance()->adicionaMensagem('Mês de competencia menor que Data de início do período de férias.');
                //tk_warning($msg);
                return;
            }

            $persiste_ferias = $this->_verificaExistenciaPeriodoFerias(
                $mes_competencia, $data_ini_per, $data_fim_per, $vinculo, $evento, $alteracao_situacao_funcional,
                $faltas_intervalo, $jornada_diaria, $this->_tipo_ferias, $nrTipoModalidadeCalc, $nrOcorrenciaCalc
            );
        }

        if ($persiste_ferias === true) {
            $this->persist();
        }

    }


    /**
     * Enter description here ...
     *
     * @param TkDateTime             $mes_competencia              Mes de competencia.
     * @param TkDateTime             $data_ini_per                 Data de inicio do periodo de ferias.
     * @param TkDateTime             $data_fim_per                 Data fim do periodo de ferias.
     * @param GpevinculomEntity      $vinculo                      Objeto Vinculo.
     * @param FpaeventomEntity       $evento                       Objeto Evento.
     * @param GpealtesitufuncEntity  $alteracao_situacao_funcional Objeto Alteracao da Situacao Funcional.
     * @param EventosFaltasIntervalo $faltas_intervalo             Objeto Eventos Faltas Intervalo.
     * @param integer                $jornada_diaria               Valor da jornada diaria.
     * @param FpatipoferiasEntity    $obj_tipo_ferias              Instancia de FpatipoferiasEntity.
     * @param integer                $nrTipoModalidadeCalc         Numero do Tipo de Modalidade de Calculo.
     * @param integer                $nrOcorrenciaCalc             Numero de Ocorrencia de Calculo.
     *
     * @return void|boolean
     *
     * @author alessandrobartels 30/11/2011
     *         alessandrobartels - 13/02/2012 - Parametrizado $nrTipoModalidadeCalc, $nrOcorrenciaCalc.
     */
    private function _verificaExistenciaPeriodoFerias(
        TkDateTime $mes_competencia, TkDateTime $data_ini_per, TkDateTime $data_fim_per, GpevinculomEntity $vinculo,
        FpaeventomEntity $evento, GpealtesitufuncEntity $alteracao_situacao_funcional,
        EventosFaltasIntervalo $faltas_intervalo, $jornada_diaria, FpatipoferiasEntity $obj_tipo_ferias,
        $nrTipoModalidadeCalc, $nrOcorrenciaCalc
    ){
        $achou_periodo_ferias = false;
        $persiste_ferias = false;
        $data_ini_anterior = new TkDateTime(TkDateTime::DUMMY_DATE, TkDateTime::FORMAT_DATE);
        $data_fim_anterior = new TkDateTime(TkDateTime::DUMMY_DATE, TkDateTime::FORMAT_DATE);

        $nr_tipoferias = $obj_tipo_ferias->getNrtipoferias();
        $nr_vinculo = $vinculo->getNrvinculom();

        /*
         * Faz loop a partir da data base projetando o periodo aquisitivo ate que o periodo aquisitivo contemple
         * o mes de competencia. Neste caso verifica se o periodo ja existe no FpaferiasEntity e cria-o caso contrario.
         * Periodos anteriores a competencia nao sao gerados. Eles deveriao ter sido migrados ou ja teriam sido
         * tratados pelo sistema
         */
        while (!$achou_periodo_ferias) {
            $ferias = $this->_retornaPeriodoFeriasSeExiste($nr_tipoferias, $data_ini_per);
            if ($ferias && ($ferias->getIdcontrolferias() == self::IDCONTROLFERIAS_ABERTO         ||
                            $ferias->getIdcontrolferias() == self::IDCONTROLFERIAS_GOZADO_PARCIAL ||
                            $ferias->getIdcontrolferias() == self::IDCONTROLFERIAS_ADQUIRIDO)
            ) {
                /**BORBA/JORGE/PAULO 11/11/2013
                 * Datas de inicio e fim devem retroceder em 1 mês para resolver a questão de faltas dentro da competencia
                 * porém após a data de fim por naõ ter ainda o controle de qual data exatamente ocorreu a falta.
                 */
                $dt_ini_per_falta = clone $data_ini_per;
                $dt_fim_per_falta = clone $data_fim_per;
                /**BORBA/JORGE/PAULO 11/11/2013 * Seta o primeiro dia do mes */
                $dt_ini_per_falta->setDay(1);
                $dt_fim_per_falta->subMonth(1);
//                tk_warning($dt_ini_per_falta->toString('dd/MM/YYYY').' ini  |  fim '.$dt_fim_per_falta->toString('dd/MM/YYYY'));

                $data_fim_anterior->setDate($data_ini_per)->subDay(1);
                $persiste_aux = $this->_verificaAfastamentosFaltas(
                    $vinculo, $ferias, $evento, $alteracao_situacao_funcional, $faltas_intervalo, $dt_ini_per_falta,
                    $dt_fim_per_falta, $jornada_diaria, $nr_tipoferias, $nrTipoModalidadeCalc,
                    $nrOcorrenciaCalc
                );

                if ($persiste_aux === true) {
                    $persiste_ferias = true;
                }
            }
            if ($mes_competencia->isLater($data_fim_per) === true) {
                if ($ferias && ($ferias->getIdcontrolferias() == self::IDCONTROLFERIAS_ABERTO)) {
                    $ferias->setIdcontrolferias(self::IDCONTROLFERIAS_ADQUIRIDO);
                    $this->addItem($ferias);

                    $persiste_ferias = true;
                }
                // Pega o proximo periodo.                $data_ini_anterior->setDate($data_ini_per);
                $data_ini_per->setDate($data_fim_per->addDay(1));
                //BORBA 10/12/2013 Misteriosamente estava fora do else
                // Soma o periodo e volta 1 dia para determinar o fim do periodo aquisitivo.
                $data_fim_per->addMonth($this->_qtd_meses_periodo_aquisitivo)->subDay(1);
            } else {

                if (!$ferias) {

                    $this->setNrvinculom($nr_vinculo);
                    $this->setNrtipoferias($nr_tipoferias);
                    $this->setNrorg($this->_nrOrg);
                    $this->setDtiniaquisicao($data_ini_per->toString());
                    $this->setDtfimaquisicao($data_fim_per);
                    $this->setIdcontrolferias(self::IDCONTROLFERIAS_ABERTO);
                    $this->setQtmesafastament(0);
                    $this->setQtfaltasperiodo(0);
                    $this->setQtdiasmaxfe($obj_tipo_ferias->historico->getQtdiasmaxfe());
                    $this->setQtdiasmaxabon($obj_tipo_ferias->historico->getQtdiasmaxabon());

                    $this->addItem($this);
                    $ferias = $this;
                    $persiste_ferias = true;
                }

                $achou_periodo_ferias = true;
            }

        }

        return $persiste_ferias;
    }


    /**
     * Verifica afastamentos e faltas.
     *
     * @param GpevinculomEntity      $vinculo                      Vinculo.
     * @param FpaeventomEntity       $evento                       Evento.
     * @param GpealtesitufuncEntity  $alteracao_situacao_funcional Alteracao da Situacao Funcional.
     * @param EventosFaltasIntervalo $faltas_intervalo             Faltas Intervalo.
     * @param TkDateTime             $data_ini_per                 Data de inicio de periodo.
     * @param TkDateTime             $data_fim_per                 Data final de periodo.
     * @param integer                $jornada_diaria               Jornada diaria de trabalho.
     * @param integer                $nr_tipoferias                Numero de tipo de ferias.
     * @param integer                $nrTipoModalidadeCalc         Numero do Tipo de Modalidade de Calculo.
     * @param integer                $nrOcorrenciaCalc             Numero de Ocorrencia de Calculo.
     *
     * @return boolean
     *
     * @author alessandrobartels
     *         alessandrobartels - 13/02/2012 - Parametrizado $nrTipoModalidadeCalc, $nrOcorrenciaCalc.
     */
    private function _verificaAfastamentosFaltas(
        GpevinculomEntity $vinculo, $ferias, FpaeventomEntity $evento, GpealtesitufuncEntity $alteracao_situacao_funcional,
        EventosFaltasIntervalo $faltas_intervalo, \TkDateTime $data_ini_per, \TkDateTime $data_fim_per, $jornada_diaria,
        $nr_tipoferias, $nrTipoModalidadeCalc, $nrOcorrenciaCalc
    ){

        $persiste = false;
        //$item_ferias = $this->_retornaPeriodoFeriasSeExiste($nr_tipoferias, $data_ini_per);
        /** @var $item_ferias \FpaferiasEntity */
        $item_ferias = $ferias;
        if ($item_ferias != false) {
            $dias_faltas = $this->_totalizaDiasFaltas(
                $vinculo, $evento, $faltas_intervalo, $data_ini_per, $data_fim_per, $jornada_diaria,
                $nrTipoModalidadeCalc, $nrOcorrenciaCalc
            );
            $dias_afastados = $this->_totalizaDiasAfastamento(
                $alteracao_situacao_funcional, $data_ini_per, $data_fim_per, $this->_dias_limite_afastamento
            );
            $meses_afastados = floor($dias_afastados/30);
//            tk_warning('Dias Faltas '.$dias_faltas.'   Dias Afast '.$dias_afastados.'  Meses Afast '.$meses_afastados);
            //$item_ferias = $this->procuraFerias($nr_ferias);
            if (($dias_faltas > 0)
                ||($dias_afastados > 0)
            ) {
                if ($this->_vai_perder_periodo === true) {
                    // Perda do periodo de ferias por afastamento.
                    /* @todo Com zero nao atualiza verificar. */
                    $item_ferias->setQtdiasmaxfe(0);
                    $item_ferias->setQtdiasmaxabon(0);

                    $item_ferias->setIdcontrolferias(self::IDCONTROLFERIAS_PERDIDO);

                    // Atualiza a data base de ferias no vinculo.
                    if ($this->_nova_data_base->equals(TkDateTime::DUMMY_DATE, TkDateTime::FORMAT_DATE) === false) {
                        $vinculo->historico->atualizaDataBaseFerias($this->_nova_data_base);
                        $vinculo->historico->adicionaPersisteHistorico($this->_mesCompetencia);
                    }
                } else {
                    $falta_ferias = $this->_falta_ferias->verificaFaltaFerias($nr_tipoferias, $dias_faltas);
                    if ($falta_ferias === null) {
                        // Perda do periodo de ferias por faltas.
                        /* @todo Com zero nao atualiza */
                        $item_ferias->setQtdiasmaxfe(0);
                        $item_ferias->setQtdiasmaxabon(0);

                        $item_ferias->setIdcontrolferias(self::IDCONTROLFERIAS_PERDIDO);
                    } else {
                        $item_ferias->setQtdiasmaxfe($falta_ferias->getQtdiasmaxfe());
                        $item_ferias->setQtdiasmaxabon($falta_ferias->getQtdiasmaxabon());
                    }
                }

                $item_ferias->setQtdiaafastament($dias_afastados);
                $item_ferias->setQtmesafastament($meses_afastados);
                $item_ferias->setQtfaltasperiodo($dias_faltas);
                /* atualizacao - nao precisa add item - alessandro 25/08/2011. */
                /*  $this->addItem($item_ferias, $item_ferias->getNrferias()); */
                $persiste = true;
            }
        }

        return $persiste;
        // Atualiza periodo antigo.
    }


    /**
     * Verifica a perda do periodo de ferias.
     *
     * @param integer $dias_afastados Dias de afastamento.
     * @param integer $dias_limite    Dias limite para perda do periodo de ferias.
     *
     * @return void
     *
     * @author alessandrobartels - 22/06/2011 - Criado para adequar ao padr�o Codesniffer.
     */
    private function _verificaPerdaPeriodo($dias_afastados, $dias_limite){
        if ((int)$dias_afastados === (int)$dias_limite) {
            $this->_vai_perder_periodo = true;
        }
    }


    /**
     * Totaliza os dias de afastamentos.
     *
     * @param GpealtesitufuncEntity $alteracao_situacao_funcional Alteracao da Situacao Funcional.
     * @param TkDateTime            $data_ini_per                 Data de inicio do periodo.
     * @param TkDateTime            $data_fim_per                 Data final do periodo.
     * @param integer               $dias_limite                  Dias limite.
     *
     * @return number
     *
     * @author alessandrobartels -
     * 		   luistavares - 20/06/2011 - Adequa��o do m�todo para o padr�o do CodeSniffer.
     *         alessandrobartels - 22/06/2011 - Retirado compara��o de perda de ferias e passado para um novo m�todo.
     */
    private function _totalizaDiasAfastamento(
        GpealtesitufuncEntity $alteracao_situacao_funcional, TkDateTime $data_ini_per,
        TkDateTime $data_fim_per, $dias_limite
    ){

        // Adiciona 1 dia na data_fim_per para verificar se a alteracao funcional tem data fim dentro do periodo.
        $data_fim_aux = new TkDateTime($data_fim_per, TkDateTime::FORMAT_DATE);
        $data_fim_aux->addDay(1);
        $afastamento = $alteracao_situacao_funcional->retornaDiasAfastamento(null, $data_ini_per, $data_fim_aux, false);
        $dias_afastados = 0;

        $this->_nova_data_base = new TkDateTime(TkDateTime::DUMMY_DATE, TkDateTime::FORMAT_DATE);
        $this->_vai_perder_periodo = false;

        for ($i=0; $i<count($afastamento); $i++) {
            if ($afastamento[$i] !== -1) {
                $sit_func = $this->_situacao_funcional->retornaSituacaoFuncional(
                    $this->_nrOrgPadrao, $this->_nrOrg, $afastamento[$i]
                );
                if ($i < (count($afastamento)-1)) {
                    if ((string)$sit_func->historico->getIdsuspferias() === GpesitufunchEntity::IDSUSPFERIAS_SIM) {
                        $dias_afastados++;
                        $this->_verificaPerdaPeriodo($dias_afastados, $dias_limite);
                        /*
                         * if ((int)$dias_afastados === (int)$dias_limite) {
                         *    $this->_vai_perder_periodo = true;
                         * }
                         */
                    }
                }

                if ($this->_vai_perder_periodo === true) {
                    if ($this->_nova_data_base->equals(TkDateTime::DUMMY_DATE, TkDateTime::FORMAT_DATE) === true) {
                        // Verifica se a proxima situacao funcional eh um dia normal.
                        if ((string)$sit_func->historico->getIdsuspferias() === GpesitufunchEntity::IDSUSPFERIAS_NAO) {
							//Marcelo 25-03-2015 - Problema da Alinutri informando que o addDay não eh uma funcao do objeto
							//Precisamos entender com a equipe Setforms a razao das duas atribuições em uma unica linha não estar funcionando						
//                            $this->_nova_data_base->set($data_ini_per)->addDay($i-1);
                              $this->_nova_data_base->set($data_ini_per);
	                 		  $this->_nova_data_base->addDay($i - 1);                        													
                        }
                    }
                }
            } else {
                if ($this->_vai_perder_periodo === true) {
                    if ($this->_nova_data_base->equals(TkDateTime::DUMMY_DATE, TkDateTime::FORMAT_DATE) === true) {					
					//Marcelo 25-03-2015 - Problema da Alinutri informando que o addDay não eh uma funcao do objeto
					//Precisamos entender com a equipe Setforms a razao das duas atribuições em uma unica linha não estar funcionando					
//                       $this->_nova_data_base->set($data_ini_per)->addDay($i-1);
                         $this->_nova_data_base->set($data_ini_per);
						 $this->_nova_data_base->addDay($i - 1);                        
                    }
                }
            }
        }

        return $dias_afastados;
    }


    /**
     * Totaliza o numero de meses de afastamentos.
     *
     * @param GpealtesitufuncEntity $alteracao_situacao_funcional Alteracao da Situacao Funcional.
     * @param TkDateTime            $data_ini_per                 Data de inicio do periodo.
     * @param TkDateTime            $data_fim_per                 Data final do periodo.
     * @param integer               $dias_afastados               Dias afastados.
     *
     * @return number
     *
     * Metodo nao utilizado - VERIFICAR.
     */
    private function _totalizaMesesAfastamento(
        GpealtesitufuncEntity $alteracao_situacao_funcional, TkDateTime $data_ini_per, TkDateTime $data_fim_per,
        $dias_afastados
    ) {

        $meses_afastados = $alteracao_situacao_funcional->retornaMesesAfastamento(
            $data_ini_per, $data_fim_per, $dias_afastados
        );
        return $meses_afastados;

    }


    /**
     * Totaliza o numero de dias de faltas / atraso.
     *
     * @param GpevinculomEntity      $vinculo              Vinculo.
     * @param FpaeventomEntity       $obj_evento           Evernto.
     * @param EventosFaltasIntervalo $faltas_intervalo     Evento Faltas Intervalo.
     * @param TkDateTime             $data_ini_per         Data de inicio do periodo.
     * @param TkDateTime             $data_fim_per         Data final do periodo.
     * @param integer                $jornada_diaria       Jornada diaria de trabalho.
     * @param integer                $nrTipoModalidadeCalc Numero do Tipo de Modalidade de Calculo.
     * @param integer                $nrOcorrenciaCalc     Numero de Ocorrencia de Calculo.
     *
     * @return number
     *
     * @author alessandrobartels
     *         alessandrobartels - 13/02/2012 - Parametrizado $nrTipoModalidadeCalc, $nrOcorrenciaCalc.
     */
    public function _totalizaDiasFaltas(
        GpevinculomEntity $vinculo, FpaeventomEntity $obj_evento, EventosFaltasIntervalo $faltas_intervalo,
        TkDateTime $data_ini_per, TkDateTime $data_fim_per, $jornada_diaria, $nrTipoModalidadeCalc, $nrOcorrenciaCalc
    ) {
        $lista_eventos = $faltas_intervalo->calcula(
            $vinculo, $data_ini_per, $data_fim_per, $nrTipoModalidadeCalc, $nrOcorrenciaCalc, $this->_nrOrgPadrao
        );
        $total_dias = 0;
        foreach ($lista_eventos as $nr_evento=> $item_evento) {
            $evento = $obj_evento->retornaEvento($nr_evento);
            $tipo_referencia = $evento->historico->getIdunidrefereven();
            $id_faltas_ferias = $evento->historico->getIdfaltasferias();

            /* @todo mudar constante */
            $total_aux = number_format(
                FpaeventohEntity::compatibilizaReferenciaDias($item_evento['REF'], $tipo_referencia, $jornada_diaria), 2
            );

            switch ($id_faltas_ferias){
            case FpaeventohEntity::IDFALTASFERIAS_SOMA:		$total_dias += $total_aux; break;
            case FpaeventohEntity::IDFALTASFERIAS_SUBTRAI: 	$total_dias -= $total_aux; break;
            }
        }

        // Retorna dias de faltas / atraso.  Arredonda para baixo.
        return  floor($total_dias);
    }


    /**
     * Inclui registro de ferias e retorna o numero de ferias.
     *
     * @param integer $nrOrg              Numero da organizacao de trabalho.
     * @param integer $nrVinculom         Numero do vinculo.
     * @param string  $dtIniAquisicao     Data de inicio de aquisicao de periodo.
     * @param string  $dtFimAquisicao     Data fim de aquisicao de periodo.
     * @param integer $nrTipoFerias       Numero do tipo de ferias.
     * @param integer $qtdFaltas          Numero de quantidade de dias de faltas.
     * @param integer $qtdDiasMaxFerias   Numero de quantidade maxima de dias de ferias.
     * @param integer $qtdDiasMaxAbono    Numero de quantidade minima de dias de ferias.
     * @param integer $qtdDiasAfastamento Numero de quantidade de dias de afastamento.
     * @param string  $idControlFerias    Identificador de controle de ferias.
     *
     * @return integer
     *
     * @author alessandrobartels - 13/10/2011
     */
    public function incluiFerias(
        $nrOrg, $nrVinculom, $dtIniAquisicao, $dtFimAquisicao, $nrTipoFerias, $qtdFaltas, $qtdDiasMaxFerias,
        $qtdDiasMaxAbono, $qtdDiasAfastamento, $idControlFerias
    ){
        $ferias = new self();
        $ferias->setNrorg($nrOrg);
        $ferias->setNrvinculom($nrVinculom);
        $ferias->setDtiniaquisicao($dtIniAquisicao);
        $ferias->setDtfimaquisicao($dtFimAquisicao);
        $ferias->setNrtipoferias($nrTipoFerias);
        $ferias->setQtfaltasperiodo($qtdFaltas);
        $ferias->setQtdiasmaxfe($qtdDiasMaxFerias);
        $ferias->setQtdiasmaxabon($qtdDiasMaxAbono);
        $ferias->setQtdiaafastament($qtdDiasAfastamento);
        $ferias->setIdcontrolferias($idControlFerias);

        $ferias->persist();

        $nrFerias = $ferias->getNrferias();
        unset($ferias);
        return $nrFerias;
    }

    public function setFerias($nrorg, $nrvinculom, $dtiniaquisicao, $dtfimaquisicao, $nrtipoferias, $qtfaltas,
                                 $qtdiasmaxferias, $qtdiasmaxabono, $qtdiasafastamento, $idcontroleferias)
    {
        $this->setNrorg($nrorg);
        $this->setNrvinculom($nrvinculom);
        $this->setDtiniaquisicao($dtiniaquisicao);
        $this->setDtfimaquisicao($dtfimaquisicao);
        $this->setNrtipoferias($nrtipoferias);
        $this->setQtfaltasperiodo($qtfaltas);
        $this->setQtdiasmaxfe($qtdiasmaxferias);
        $this->setQtdiasmaxabon($qtdiasmaxabono);
        $this->setQtdiaafastament($qtdiasafastamento);
        $this->setIdcontrolferias($idcontroleferias);
    }

    public static function getByNrorgVinculomTipoferiasData($nrorg, $nrvinculom, $tpferias, $dtiniaquisicao)
    {
        $fpaferias = self::getEntityByWhere(
            'NRORG = ? AND NRVINCULOM = ? AND NRTIPOFERIAS = ? AND DTINIAQUISICAO = ?',
            array($nrorg, $nrvinculom, $tpferias, $dtiniaquisicao)
        );
        return $fpaferias;
    }

    /**
     * @return FpagozoferiasEntity[]|mixed
     */
    public function getGozoFerias() {
        if (!isset($this->_gozosFerias)) {
            $this->_gozosFerias = Fpagozoferias::getObjectsByWhere(
                'NRORG = ? AND NRFERIAS = ?', array($this->getNrorg(), $this->getNrferias())
            );
        }
        return $this->_gozosFerias;
    }

}

