/******************************************************************************/
/* Sistemas Integrados TECNISA                                                */
/******************************************************************************/
/* Nome: MES_FATURAMENTO                                                      */
/*                                                                            */
/* Descrio:                                                                 */
/* Calcula datas de faturamento para um determinado mes baseando-se na        */
/* Periodicidade de faturamento e nos Feriados.                               */
/*                                                                            */
/* Criao:                                                                   */
/* 30/05/2003 - Gustavo Nosseis dos Santos                                    */
/******************************************************************************/
/* PARAMETROS:                                                                */
/*                                                                            */
/* P_CDFILIAL     - Cdigo da Filial                                          */
/* P_CDCLIENTE    - Cdigo do Cliente                                         */
/* P_CDPERIENTR   - Cdigo da Periodicidade de Entrega                        */
/* P_DTINICIAL    - Data do Inicio do Mes                                     */
/* P_NRMESES      - Numero de meses a serem calculados                        */
/******************************************************************************/
/* @C:\WIN_PRJ\SQL\END_CONV\ORACLE\8681_VMNSTP_MES_FATURAMENTO.SQL            */
/******************************************************************************/
CREATE OR REPLACE
PROCEDURE MES_FATURAMENTO                                                                                                                                                                                                                                       
  (P_CDFILIAL     IN VARCHAR2,                                                                                                                                                   P_CDCLIENTE    IN VARCHAR2,                                                                           
   P_CDPERIENTR   IN VARCHAR2,
   P_DTINICIAL    IN DATE,                                                                                                                                                                                                                                      
   P_IDDIANUTIL   IN VARCHAR2) IS                                                                                                                                                                                                                                   
   V_CONTAUXI   NUMBER;                                                                                                                                                                                                                                         
   V_DIENTREGA  VARCHAR2(2);                                                                                                                                                                                                                                    
   V_DTDIAENTR  DATE;                                                                                                                                                                                                                                           
   V_IDDIASEM   NUMBER;                                                                                                                                                                                                                                         
   V_DTENTRAUX  DATE;                                                                                                                                                                                                                                           
   V_PRIMVEZ    NUMBER;                                                                                                                                                                                                                                         
   V_DIPRIMVEZ  VARCHAR2(2);                                                                                                                                                                                                                                    
   V_JAEXISTE   NUMBER;                                                                                                                                                                                                                                         
/*  Cursor que contem todos os dias de faturamento de determinada periodicidade  */                                                                                                                                                                           
   CURSOR C_DIAS_ENTR IS                                                                                                                                                                                                                                        
      SELECT DIENTREGA                                                                                                                                                                                                                                          
      FROM DIASENTR                                                                                                                                                                                                                                             
      WHERE                                                                                                                                                                                                                                                     
        CDPERIENTR = P_CDPERIENTR                                                                                                                                                                                                                               
      ORDER BY DIENTREGA;                                                                                                                                                                                                                                       
BEGIN                                                                                                                                                                                                                                                           
    V_DTENTRAUX     := P_DTINICIAL;                                                                                                                                                                                                                             
    OPEN C_DIAS_ENTR;                                                                                                                                                                                                                                           
    LOOP                                                                                                                                                                                                                                                        
      FETCH C_DIAS_ENTR                                                                                                                                                                                                                                         
      INTO V_DIENTREGA;                                                                                                                                                                                                                                         
      EXIT WHEN C_DIAS_ENTR%NOTFOUND;                                                                                                                                                                                                                           
     V_DTDIAENTR := (P_DTINICIAL - 1) + V_DIENTREGA;                                                                                                                                                                                                            
     IF V_DTDIAENTR > LAST_DAY(P_DTINICIAL) THEN                                                                                                                                                                                                                
        V_DTDIAENTR := LAST_DAY(P_DTINICIAL);                                                                                                                                                                                                                   
     END IF;                                                                                                                                                                                                                                                    
/*  Calcula uma data de faturamento anterior a uma data nao util  */                                                                                                                                                                                            
        IF P_IDDIANUTIL = 'S' THEN                                                                                                                                                                                                                              
          V_CONTAUXI := 1;                                                                                                                                                                                                                                      
           WHILE V_CONTAUXI > 0 LOOP                                                                                                                                                                                                                            
             SELECT COUNT(*) INTO V_CONTAUXI                                                                                                                                                                                                                    
             FROM FERIFILI                                                                                                                                                                                                                                      
             WHERE                                                                                                                                                                                                                                              
             CDFILIAL = P_CDFILIAL AND                                                                                                                                                                                                                          
             DTFERIFILI = V_DTDIAENTR;                                                                                                                                                                                                                          
             IF V_CONTAUXI > 0 THEN                                                                                                                                                                                                                             
                V_DTDIAENTR := V_DTDIAENTR - 1;                                                                                                                                                                                                                 
             END IF;                                                                                                                                                                                                                                            
           END LOOP;                                                                                                                                                                                                                                            
         END IF;                                                                                                                                                                                                                                                
/*  Verifica se j existe a data de faturamento que foi calculada por um dia no util  */                                                                                                                                                                           
        SELECT COUNT(*) INTO V_JAEXISTE                                                                                                                                                                                                                         
        FROM CALEFATURVMN                                                                                                                                                                                                                                           
        WHERE                                                                                                                                                                                                                                                   
          CDCLIENTE  = P_CDCLIENTE AND
          CDFILIAL   = P_CDFILIAL  AND
          DTFATURVMN = V_DTDIAENTR;
/*  Se j existir a data pula  */
        IF V_JAEXISTE = 0 THEN
/*  Verifica se ja existe alguma data de faturacimento calculada para incluir a  */
/*  data anterior ao proximo faturamento  */
           SELECT COUNT(*) INTO V_PRIMVEZ
           FROM CALEFATURVMN
           WHERE
           CDCLIENTE  = P_CDCLIENTE AND
           CDFILIAL   = P_CDFILIAL  AND
           DTFATURVMN = V_DTENTRAUX;
/*  Inclui a data de faturamento calculada no calendario de faturamento  */
           INSERT INTO CALEFATURVMN
             (CDCLIENTE,CDFILIAL,DTFATURVMN)
             VALUES
             (P_CDCLIENTE,P_CDFILIAL,V_DTDIAENTR);
           V_DTENTRAUX := V_DTDIAENTR;
        END IF;
    END LOOP;
    CLOSE C_DIAS_ENTR;
END;
/

